/* Perfect Sticks player. (C) MM by me */
#include "sticks.h"
#include <stdio.h>
#include <stdlib.h>

typedef struct {
  int row, lo, hi;
} pile;

int current;            /* The starting position */
int winning[1024];      /* Marks all the winning configurations */
pile winningmove[1024]; /* What move one must actually make to win */
pile board[11][50];     /* Stores the current board configuration */
int nboards[11];        /* The number of boards of each distinct size */
int turn;               /* Whose turn it is, 1=program, 0=module */
int n;	                /* Number of rows */

void insertpile(int row, int lo, int hi)
{
  int size=hi-lo+1;
  board[size][++nboards[size]].row=row;
  board[size][nboards[size]].lo=lo;
  board[size][nboards[size]].hi=hi;
  current^=1<<(size-1);
}

void deletepile(int size, int index)
{
  board[size][index]=board[size][nboards[size]--];
  current^=1<<(size-1);
}

void lookuppile(int row, int lo, int hi, int *size, int *index)
{
  for (*size=hi-lo+1; *size<=10; (*size)++)
    for (*index=1; *index<=nboards[*size]; (*index)++)
      if (board[*size][*index].row==row &&
	  board[*size][*index].lo<=lo &&
	  board[*size][*index].hi>=hi)
	return;
}

void readdata(void)
{
  FILE *f=fopen("STICKS.IN", "r");
  int i, x;
  current=0;

  for (i=1; i<=10; nboards[i++]=0);
  fscanf(f, "%d", &n);
  for (i=1; i<=n; i++)
    {
      fscanf(f, "%d", &x);
      insertpile(i, 1, x);
    }
  fscanf(f, "%d", &turn);
  fclose(f);
}

void expand(void)
{
  int i, j, k, l, m;

  winning[0]=0;
  for (i=1; i<1024; i++)
    {
      winning[i]=0;
      for (j=1; j<=10; j++)
	if (i&(1<<(j-1))) /* Configuration i contains a row of size j */
	  {
	    for (l=max(j-3,0); l<=max(j-1,0); l++) /* l is the number of sticks left */
	      for (m=0; m<=l/2; m++) /* we split l in m and l-m */
		{
		  k=i^(1<<(j-1));
		  if (m>0) k^=1<<(m-1);
		  if (l-m>0) k^=1<<(l-m-1);
		  if (!winning[k])
		    {
		      winning[i]=1;
		      winningmove[i].row=j;
		      winningmove[i].lo=m+1;
		      winningmove[i].hi=m+j-l;
		    }
		}
	  }
    }
}

void makeamove(int size, int index, int lo, int hi)
{
  pile p=board[size][index];
  deletepile(size, index);
  if (p.lo<lo) insertpile(p.row, p.lo, lo-1);
  if (p.hi>hi) insertpile(p.row, hi+1, p.hi);
}

void computermove(void)
{
  int row, lo, hi, size, index;

  getMove(&row, &lo, &hi);
/*  printf(">>Computer move: %d %d %d\n", row, lo, hi);*/
  lookuppile(row, lo, hi, &size, &index);
  makeamove(size, index, lo, hi);
}

void programmove(void)
{
  pile p, wm;
  int size;

  if (winning[current])
    {
      wm=winningmove[current];
      p=board[wm.row][1];
/*      printf(">>Human winning move: %d %d %d\n", p.row, p.lo+wm.lo-1, p.lo+wm.hi-1);*/
      putMove(p.row, p.lo+wm.lo-1, p.lo+wm.hi-1);
      makeamove(wm.row, 1, p.lo+wm.lo-1, p.lo+wm.hi-1);
    }
  else {
    size=0;
    while (!nboards[size]) size++;
    p=board[size][1];
/*    printf(">>Human losing move: %d %d %d\n", p.row, p.lo, p.lo);*/
    putMove(p.row, p.lo, p.lo);
    makeamove(size, 1, p.lo, p.lo);
  }
}

int main(void)
{
  expand();
  readdata();
  if (turn) computermove();
  while (1) {
    programmove();
    computermove();
  }
}
