/*
Author:        Silviu-Ionuţ Gănceanu
Description:   DP with memoization.
Time:          O(N^4)
Memory:        O(N^3)
*/

#include <cstdio>
#include <cassert>
#include <cstring>
#include <climits>
#include <algorithm>
#include <vector>
#include <utility>

using namespace std;

const int MIN_N = 1;
const int MAX_N = 200;
const int MIN_A = 1;
const int MAX_A = 1000000000;

int N, A;
vector<pair<int, int> > points;

void read_data() {
  int x, y;
  FILE *f = fopen("photo.in", "rt");
  fscanf(f, "%d %d", &N, &A);
  assert(MIN_N <= N && N <= MAX_N);
  assert(MIN_A <= A && A <= MAX_A);
  points.reserve(N);
  for (int i = 0; i < N; ++i) {
    fscanf(f, "%d %d", &x, &y);
    assert(x >= 0);
    assert(A >= y && y >= 1);
    points.push_back(make_pair(x, y));
  }
  fclose(f);
}

typedef unsigned char cache_t;
vector<int> Ys;
cache_t cache[MAX_N][MAX_N][MAX_N];
#define XX first
#define YY second

cache_t best(int left, int right, int up) {
//  printf("left = %d, right = %d, up = %d\n", left, right, up);
  cache_t &crt = cache[left][right][up];
  if (crt != 255) {
    return crt;
  }
  int max_y = -1;
  int max_x = -1;
  int min_x = -1;
  for (int i = left; i <= right; ++i) {
    if (points[i].YY >= Ys[up]) {
      if (max_y == -1) {
        max_y = points[i].YY;
        min_x = points[i].XX;
        max_x = points[i].XX;
      } else {
        max_y = max(max_y, points[i].YY);
        min_x = min(min_x, points[i].XX);
        max_x = max(max_x, points[i].XX);
      }
    }
  }
//  printf("max_y = %d, min_x = %d, max_x = %d\n", max_y, min_x, max_x);
  // No point in the range.
  if (max_y == -1) {
    return crt = 0;
  }
  // All points can be covered with one rectangle.
  long long area = (long long) (max_x - min_x) * (long long) max_y;
  if (area <= A) {
    return crt = 1;
  }

  crt = N + 1;

  // Draw rectangle.
  int new_y = A / (max_x - min_x);
  int new_left = -1, new_right = -1, new_up = -1;
  for (int i = left; i <= right; ++i) {
    if (points[i].YY > new_y) {
      if (new_left == -1) new_left = i;
      new_right = i;
    }
  }
  for (int i = 0; i < (int) Ys.size(); ++i) {
    if (Ys[i] > new_y) {
      new_up = i;
      break;
    }
  }
  assert(new_up != -1 && new_left != -1 && new_right != -1);
  if (new_left != left || new_right != right || new_up != up) {
    cache_t tmp = best(new_left, new_right, new_up) + 1;
    if (tmp < crt) {
      crt = tmp;
    }
  }

  // Split vertically.
  for (int i = left; i < right; ++i) {
    cache_t tmp = best(left, i, up) + best(i + 1, right, up);
    if (tmp < crt) {
      crt = tmp;
    }
  }
  return crt;
}

bool cmpf(const pair<int, int> &a, const pair<int, int> &b) {
  if (a.XX - b.XX)
    return a.XX < b.XX;
  return a.YY > b.YY;
}

bool equalf(const pair<int, int> &a, const pair<int, int> &b) {
  return a.XX == b.XX;
}

void solve_and_print() {
  sort(points.begin(), points.end(), cmpf);
  // If two points have the same x keep the one with the maximum y.
  points.erase(unique(points.begin(), points.end(), equalf), points.end());
  N = points.size();

  for (int i = 0; i < N; ++i) {
    Ys.push_back(points[i].YY);
  }
  sort(Ys.begin(), Ys.end());
  Ys.erase(unique(Ys.begin(), Ys.end()), Ys.end());
  printf("real N = %d\n", N);
  printf("num diff Ys = %d\n", Ys.size());
  for (int i = 0; i < (int) Ys.size(); ++i) {
    printf("%5d", Ys[i]);
  }
  printf("\n");
  for (int i = 0; i < (int) Ys.size(); ++i) {
    int count = 0;
    for (int j = 0; j < N; ++j) {
      count += (points[j].YY == Ys[i]);
    }
    printf("%5d", count);
  }
  printf("\n");

  //for (int i = 0; i < N; ++i) {
  //  printf("%3d %3d\n", points[i].XX, points[i].YY);
  //}
  //for (int i = 0; i < (int) Ys.size(); ++i) {
  //  printf("%3d\n", Ys[i]);
  //}

  memset(cache, -1, sizeof(cache));

  FILE *f = fopen("photo.out", "wt");
  fprintf(f, "%d\n", best(0, N - 1, 0));
  fclose(f);
}

int main() {
  read_data();
  solve_and_print();
  return 0;
}

