/*
Author:        Silviu-Ionuţ Gănceanu
Description:   Tries to cover the points top-bottom by choosing the rectangles
               whith the maximum height that cover the maximum number of points.
               If several options are available, toss a coin.
               Repeat the greedy 1000 times.
*/

#include <cstdio>
#include <cstring>
#include <ctime>
#include <algorithm>
#include <vector>
#include <utility>

using namespace std;

const int MAX_N = 200;

#define xx first
#define yy second
int N, A;
vector<pair<int, int> > point;

void read_data() {
  FILE *f = fopen("photo.in", "r");
  fscanf(f, "%d %d", &N, &A);
  point.reserve(N);
  int x, y;
  for (int i = 0; i < N; ++i) {
    fscanf(f, "%d %d", &x, &y);
    point.push_back(make_pair(x, y));
  }
  fclose(f);
}

bool cmpf(const pair<int, int> &a, const pair<int, int> &b) {
  if (a.xx - b.xx)
    return a.xx < b.xx;
  return a.yy > b.yy;
}

bool cmpf2(const pair<int, int> &a, const pair<int, int> &b) {
  if (a.yy - b.yy)
    return a.yy > b.yy;
  return a.xx < b.xx;
}

bool cmpf3(int a, int b) {
  return point[a].xx < point[b].xx;
}

bool equalf(const pair<int, int> &a, const pair<int, int> &b) {
  return a.xx == b.xx;
}

bool covered[MAX_N];
int width;

void cover(int left, int right) {
  //printf("left = %d, right = %d\n", left, right);
  vector<int> candidate;
  for (int i = 0; i < (int) point.size(); ++i) {
    if (covered[i] == false && point[i].xx >= point[right].xx - width &&
        point[i].xx <= point[left].xx + width) {
      candidate.push_back(i);
    }
  }
  sort(candidate.begin(), candidate.end(), cmpf3);
  //printf("candidates are:\n");
  //for (int i = 0; i < (int) candidate.size(); ++i) {
  //  printf("%d %d\n", point[candidate[i]].xx, point[candidate[i]].yy);
  //}
  int last = 0, count = 0, max_count = -1, max_pos;
  for (int i = 0; i < (int) candidate.size() && last < (int) candidate.size(); ++i) {
    while (last < (int) candidate.size() && point[candidate[last]].xx -
        point[candidate[i]].xx <= width) {
      ++last;
      ++count;
    }
    if (count > max_count) {
      max_count = count;
      max_pos = i;
    } else if (count == max_count) {
      if (rand() % 2) {
        max_pos = i;
      }
    }
    --count;
  }
  for (int i = max_pos; i < (int) candidate.size(); ++i) {
    if (point[candidate[i]].xx - point[candidate[max_pos]].xx <= width) {
      covered[candidate[i]] = true;
      //printf("%d\n", candidate[i]);
    }
  }
}

void solve_and_print() {
  // TODO(silviug): check that indeed the points are erased corectly.
  srand(time(0));
  sort(point.begin(), point.end(), cmpf);
  point.erase(unique(point.begin(), point.end(), equalf), point.end());
  //for (int i = 0; i < (int) point.size(); ++i) {
  //  printf("%d %d\n", point[i].xx, point[i].yy);
  //}
  //printf("\n\n");

  vector<pair<int, int> > old_point(point);

  int best_sol = N;
  for (int wer = 0; wer < 2000; ++wer) {
    int sol = 0;
    point = old_point;
    while (point.empty() == false) {
      sort(point.begin(), point.end(), cmpf2);
      memset(covered, 0, sizeof(covered));
      width = A / point[0].yy;
      //printf("---------\n");
      //for (int i = 0; i < (int) point.size(); ++i) {
      //  printf("%3d %3d\n", point[i].xx, point[i].yy);
      //}
      //printf("width = %d\n", width);
      int last = 0, i;
      for (i = 0; i < (int) point.size(); ++i) {
        if (point[i].yy < point[0].yy) {
          break;
        }
        if (point[i].xx - point[last].xx > width) {
          cover(last, i - 1);
          sol += 1;
          last = i;
        }
      }
      if (last <= i - 1) {
        cover(last, i - 1);
        sol += 1;
      }

      vector<pair<int, int> > new_point;
      for (int i = 0; i < (int) point.size(); ++i) {
        if (covered[i] == false) {
          new_point.push_back(point[i]);
        }
      }
      point = new_point;
    }
    best_sol = min(best_sol, sol);
  }
  FILE *f = fopen("photo.out", "w");
  fprintf(f, "%d\n", best_sol);
  fclose(f);
}

int main() {
  read_data();
  solve_and_print();
  return 0;
}

